@@ -347,28 +347,28 @@ short checkbounds()
 
    /* check window-to-user space */
 
-   lval = 2 * (long)((float) (areawin->width) / (*areawin->vscale)) +
-	(long)areawin->pcorner->x;
+   lval = 2 * (long)((float) (areawin->width) / areawin->vscale) +
+	(long)areawin->pcorner.x;
    if (lval != (long)((short)lval)) return -1;
-   lval = 2 * (long)((float) (areawin->height) / (*areawin->vscale)) +
-	(long)areawin->pcorner->y;
+   lval = 2 * (long)((float) (areawin->height) / areawin->vscale) +
+	(long)areawin->pcorner.y;
    if (lval != (long)((short)lval)) return -1;
 
    /* check user-to-window space */
 
-   lval = (long)((float)(topobject->bbox.lowerleft.x - areawin->pcorner->x) *
-	(*areawin->vscale));
+   lval = (long)((float)(topobject->bbox.lowerleft.x - areawin->pcorner.x) *
+	areawin->vscale);
    if (lval != (long)((short)lval)) return -1;
    lval = (long)areawin->height - (long)((float)(topobject->bbox.lowerleft.y -
-	areawin->pcorner->y) * (*areawin->vscale)); 
+	areawin->pcorner.y) * areawin->vscale); 
    if (lval != (long)((short)lval)) return -1;
    UTransformbyCTM(DCTM, &(topobject->bbox.lowerleft), &testpt, 1);
 
    lval = (long)((float)(topobject->bbox.lowerleft.x + topobject->bbox.width -
-	areawin->pcorner->x) * (*areawin->vscale));
+	areawin->pcorner.x) * areawin->vscale);
    if (lval != (long)((short)lval)) return -1;
    lval = (long)areawin->height - (long)((float)(topobject->bbox.lowerleft.y +
-	topobject->bbox.height - areawin->pcorner->y) * (*areawin->vscale)); 
+	topobject->bbox.height - areawin->pcorner.y) * areawin->vscale); 
    if (lval != (long)((short)lval)) return -1;
 
    return 0;
@@ -382,9 +382,9 @@ void window_to_user(short xw, short yw, XPoint *upt)
 {
   float tmpx, tmpy;
 
-  tmpx = (float)xw / (*areawin->vscale) + (float)areawin->pcorner->x;
-  tmpy = (float)(areawin->height - yw) / (*areawin->vscale) + 
-	(float)areawin->pcorner->y;
+  tmpx = (float)xw / areawin->vscale + (float)areawin->pcorner.x;
+  tmpy = (float)(areawin->height - yw) / areawin->vscale + 
+	(float)areawin->pcorner.y;
 
   tmpx += (tmpx > 0) ? 0.5 : -0.5;
   tmpy += (tmpy > 0) ? 0.5 : -0.5;
@@ -401,9 +401,9 @@ void user_to_window(XPoint upt, XPoint *wpt)
 {
   float tmpx, tmpy;
 
-  tmpx = (float)(upt.x - areawin->pcorner->x) * (*areawin->vscale);
-  tmpy = (float)areawin->height - (float)(upt.y - areawin->pcorner->y)
-	* (*areawin->vscale); 
+  tmpx = (float)(upt.x - areawin->pcorner.x) * areawin->vscale;
+  tmpy = (float)areawin->height - (float)(upt.y - areawin->pcorner.y)
+	* areawin->vscale; 
 
   tmpx += (tmpx > 0) ? 0.5 : -0.5;
   tmpy += (tmpy > 0) ? 0.5 : -0.5;
@@ -571,14 +571,14 @@ void UCopyCTM(fctm, tctm)
 
 void UMakeWCTM(Matrix *ctm)
 {
-   ctm->a *= (*areawin->vscale);
-   ctm->b *= (*areawin->vscale);
-   ctm->c = (ctm->c - (float)areawin->pcorner->x) * (*areawin->vscale);
+   ctm->a *= areawin->vscale;
+   ctm->b *= areawin->vscale;
+   ctm->c = (ctm->c - (float)areawin->pcorner.x) * areawin->vscale;
    
-   ctm->d *= -(*areawin->vscale);
-   ctm->e *= -(*areawin->vscale);
-   ctm->f = (float)areawin->height + ((float)areawin->pcorner->y - ctm->f) *
-	    (*areawin->vscale);
+   ctm->d *= -areawin->vscale;
+   ctm->e *= -areawin->vscale;
+   ctm->f = (float)areawin->height + ((float)areawin->pcorner.y - ctm->f) *
+	    areawin->vscale;
 }
 
 /*------------------------------------------------------------------------*/
@@ -1359,6 +1359,13 @@ void centerview(objinstptr tinst)
 	       / tobj->viewscale - width) / 2;
    tobj->pcorner.y = origin.y - (areawin->height
 	       / tobj->viewscale - height) / 2;
+
+   /* Copy new position values to the current window */
+
+   if (tobj == topobject) {
+      areawin->pcorner = tobj->pcorner;
+      areawin->vscale = tobj->viewscale;
+   }
 }
 
 /*-----------------------------------------------------------*/
@@ -1383,7 +1390,8 @@ void zoomview(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE || eventmode == COPY_MODE ||
 	 eventmode == MOVE_MODE || eventmode == CATALOG_MODE ||
-	eventmode == FONTCAT_MODE || eventmode == EFONTCAT_MODE) {
+	eventmode == FONTCAT_MODE || eventmode == EFONTCAT_MODE ||
+	eventmode == CATMOVE_MODE) {
 
       centerview(areawin->topinstance);
       invalidate_graphics(topobject);
@@ -1834,11 +1842,12 @@ void strokepath(XPoint *pathlist, short number, short style, float width)
       if (style & (DASHED | DOTTED)) {
          SetDashes(dpy, areawin->gc, 0, dashstring, 2);
          SetLineAttributes(dpy, areawin->gc, tmpwidth, LineOnOffDash,
-		CapButt, JoinMiter);
+		CapButt, (style & SQUARECAP) ? JoinMiter : JoinBevel);
       }
       else
          SetLineAttributes(dpy, areawin->gc, tmpwidth, LineSolid,
-		(style & SQUARECAP) ? CapProjecting : CapRound, JoinMiter);
+		(style & SQUARECAP) ? CapProjecting : CapRound,
+		(style & SQUARECAP) ? JoinMiter : JoinBevel);
 
       /* draw the spline and close off if so specified */
       DrawLines(dpy, areawin->window, areawin->gc, pathlist,
@@ -2166,21 +2175,34 @@ void TopDoLatex()
    int tx, ty, width, height;
    XPoint origin;
    Boolean checklatex = FALSE;
-   char filename[50];
+   char filename[100], extend[10], *dotptr;
 
    UDoLatex(areawin->topinstance, 0, NULL, 1.0, 0, 0, &checklatex);
 
    if (checklatex == FALSE) return;	/* No LaTeX labels to write */
 
+   /* Handle cases where the file might have a ".eps" extension.	*/
+   /* Thanks to Graham Sheward for pointing this out.			*/
+
    if (xobjs.pagelist[areawin->page]->filename)
-      sprintf(filename, "%s.tex", xobjs.pagelist[areawin->page]->filename);
+      sprintf(filename, "%s", xobjs.pagelist[areawin->page]->filename);
    else
-      sprintf(filename, "%s.tex",
+      sprintf(filename, "%s",
 		xobjs.pagelist[areawin->page]->pageinst->thisobject->name);
+
+   if ((dotptr = strchr(filename + strlen(filename) - 4, '.')) == NULL) {
+       dotptr = filename + strlen(filename);
+       sprintf(dotptr, ".ps");
+   }
+   strcpy(extend, dotptr);
+   strcpy(dotptr, ".tex");
+
    f = fopen(filename, "w");
 
-   fprintf(f, "%% XCircuit output \"%s\" for LaTeX input from %s.ps\n",
-		filename, xobjs.pagelist[areawin->page]->filename);
+   *dotptr = '\0';
+
+   fprintf(f, "%% XCircuit output \"%s.tex\" for LaTeX input from %s%s\n",
+		filename, filename, extend);
    fprintf(f, "\\def\\putbox#1#2#3{\\makebox[0in][l]{\\makebox[#1][l]{}"
 		"\\raisebox{\\baselineskip}[0in][0in]"
 		"{\\raisebox{#2}[0in][0in]{#3}}}}\n");
@@ -2190,13 +2212,7 @@ void TopDoLatex()
    fprintf(f, "\\def\\midbox#1{\\raisebox{-0.5\\baselineskip}[0in][0in]{#1}}\n");
    fprintf(f, "\\begin{flushleft}\n");
    
-   /* Handle cases where the file might have a ".eps" extension.	*/
-   /* Thanks to Graham Sheward for pointing this out.			*/
-   filename[strlen(filename) - 4] = '\0';
-   if (strchr(filename, '.') == NULL)
-       sprintf(filename + strlen(filename), ".ps");
-
-   fprintf(f, "\\epsfig{file=%s}\\\\\n", filename);
+   fprintf(f, "\\epsfig{file=%s%s}\\\\\n", filename, extend);
 
    psscale = getpsscale(xobjs.pagelist[areawin->page]->outscale,
 		areawin->page);
@@ -2216,5 +2232,7 @@ void TopDoLatex()
 
    fprintf(f, "\\end{flushleft}\n");
    fclose(f);
+
+   Wprintf("Wrote auxiliary file %s.tex", filename);
 }
 
