@@ -309,7 +309,8 @@ enum editmode {
   EARC_MODE,		/* Editing an existing arc */
   ESPLINE_MODE,		/* Editing an existing spline */
   EPATH_MODE,		/* Editing an existing path */
-  ASSOC_MODE		/* Choosing an associated schematic or symbol */
+  ASSOC_MODE,		/* Choosing an associated schematic or symbol */
+  CATMOVE_MODE		/* Moving objects in or between libraries */
 };
 
 /*----------------------------------------------------------------------*/
@@ -364,6 +365,19 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define PARAM_START	17  /* bounds a parameter; data = param key	*/
 #define PARAM_END	18  /* bounds a parameter; no data 		*/
 
+/* Actions translated to keystates */
+
+#define TEXT_RETURN	19
+#define TEXT_HOME	20
+#define TEXT_END	21
+#define TEXT_SPLIT	22
+#define TEXT_DOWN	23
+#define TEXT_UP		24
+#define TEXT_LEFT	25
+#define TEXT_RIGHT	26
+#define TEXT_DELETE	27
+#define TEXT_DEL_PARAM	28
+
 #define SPECIAL		63  /* used only when called from menu		*/
 #define NULL_TYPE	255 /* used as a placeholder			*/
 
@@ -1153,6 +1167,7 @@ typedef struct {
 typedef struct _keybinding *keybindingptr;
 
 typedef struct _keybinding {
+   xcWidget window;		/* per-window function, or NULL */
    int keywstate;
    int function;
    short value;
@@ -1263,7 +1278,11 @@ typedef struct {
 /* all the critical data needed by the drawing window 			*/
 /*----------------------------------------------------------------------*/
 
-typedef struct {
+typedef struct _windowdata *XCWindowDataPtr;
+
+typedef struct _windowdata {
+
+   XCWindowDataPtr next;	/* next window in list */
 
    /* widgets and X11 parameters */
    xcWidget	area;
@@ -1276,8 +1295,8 @@ typedef struct {
    /* global page parameters */
    short	width, height;
    short	page;
-   float	*vscale;	/* pointer to proper scale */
-   XPoint	*pcorner;	/* pointer to page position */
+   float	vscale;		/* proper scale */
+   XPoint	pcorner;	/* page position */
 
    /* global option defaults */
    float	textscale;
@@ -1286,8 +1305,8 @@ typedef struct {
    short	psfont;
    short	justify;
    u_short	style;
-   short	filter;		/* selection filter */
    int		color;
+   short	filter;		/* selection filter */
    Boolean	manhatn;
    Boolean	boxedit;
    Boolean	snapto;
@@ -1297,10 +1316,8 @@ typedef struct {
    Boolean	axeson;
    Boolean	invert;
    Boolean	mapped;		/* indicates if window is drawable */
-   Boolean	filefilter;	/* Is the file list filtered? */
    char		buschar;	/* Character indicating vector notation */
    Boolean	editinplace;
-   Boolean	schemon;
    Boolean	pinpointon;
    Boolean	attachto;
 #ifdef HAVE_XPM
@@ -1322,7 +1339,7 @@ typedef struct {
    short	editpart, editsubpart;
    char		*lastbackground;
    Cursor	*defaultcursor;
-} XCWindowData, *XCWindowDataPtr;
+} XCWindowData;
 
 /* Record for undo function */ 
 
@@ -1334,6 +1351,7 @@ typedef struct _undostack {
    u_int type;		/* type of event */
    short idx;		/* counter for undo event */
    objinstptr thisinst;	/* instance of object in which event occurred */
+   XCWindowData *window;/* window in which event occurred */
    int idata;		/* simple undedicated integer datum */
    char *undodata;	/* free space to be malloc'd */
 			/* (size dependent on "type") */
@@ -1365,6 +1383,7 @@ typedef struct {
    xcIntervalId	timeout_id;
    int		save_interval;
    short	do_replace;
+   Boolean	filefilter;	/* Is the file list filtered? */
    u_short	new_changes;
    short	numlibs;
    short	pages;
@@ -1376,8 +1395,7 @@ typedef struct {
    objinstptr   *libtop;
    Imagedata	*imagelist;
    short	images;
-   XCWindowData *windowlist;
-   short	windows;
+   XCWindowData *windowlist;	/* linked list of known windows */
 } Globaldata;
 
 /*----------------------------------------------------------------------*/
